'' Test program for Vinculum and Propeller Demo Board

CON
  _clkmode  = xtal1 + pll16x
  _xinfreq  = 5_000_000

  RESET     = 0                   ' Vinculum /Reset
  SPI_CLK   = 1                   ' Vinculum Clock
  SPI_DATA  = 2                   ' Vinculum Data
  SPI_CS    = 3                   ' Vinculum Select
  EOL       = 13                  ' End of line (return)

  cols = 64
  rows = 48
  tiles = cols * rows

  spacetile = $8000 + $20 << 6

VAR
  long  col, row, color
  long  array[tiles/2]

OBJ
  key : "keyboard"
  vga : "vga_1024x768_Driver"

PUB start | t, s
  vga.start(16, @array, @vgacolors, 0, 0, 0)
  key.start(26,27)                ' Keyboard pins for Demo Board
  print($100)
  print_str(string("Vinculum Test Program",EOL))
  outa[RESET]~                    ' Reset device
  dira[RESET]~~
  outa[SPI_CS]~                   ' Select normally low
  dira[SPI_CS]~~
  outa[SPI_CLK]~                  ' Clock normally low
  dira[SPI_CLK]~~
  dira[SPI_DATA]~                 ' Data initially input
  outa[RESET]~~
  s := 0
  waitcnt(clkfreq / 4 + cnt)      ' Wait for 250ms initialization
  repeat
    if s == 0                     ' Buffer not full in Vinculum?
      s := key.key                ' Look for typed character
    if s > 0                      ' Character pending?
      if (transfer(false,false,s) & 1) == 0
        print(s)                  ' If transmitted ok, echo to display
        s := 0                    ' Allow another
    if ((t := transfer(true,false,0)) & 1) == 0
      t := t >> 1 & $FF           ' If character read from Vinculum
      if t & $7F => " " or t == EOL
        print(t)                  ' Display all displayable characters
      else
        print("{")                ' Or display hex code of others
        print_hex(t,2)
        print("}")

PRI transfer(read,status,data) | i, mask
'' Transfer data to/from Vinculum USB Host in SPI mode
  data := $800 | $400 & read | $200 & status | data << 1
  mask := $800
  outa[SPI_CLK]~~                 ' At least one clock prior to selecting
  outa[SPI_CLK]~
  outa[SPI_CS]~~                  ' Select chip
  repeat i from 1 to 12           ' Start, direction, address, 8 data, status
    dira[SPI_DATA] := i < 4 or i < 12 and not read
    outa[SPI_DATA] := (data & mask) > 0
    result |= mask & (ina[SPI_DATA] > 0)
    outa[SPI_CLK]~~               ' Data valid on positive edge of clock
    dira[SPI_DATA]~               ' Let data line float after hold time
    outa[SPI_CLK]~
    mask >>= 1
  outa[SPI_CS]~                   ' Deselect chip
  outa[SPI_CLK]~~                 ' At least one clock after deselecting
  outa[SPI_CLK]~

PRI print_str(stringptr)
'' Print a zero-terminated string
  repeat strsize(stringptr)
    print(byte[stringptr++])

PRI print_dec(value) | i
'' Print a decimal number
  if value < 0
    -value
    print("-")
  i := 1_000_000_000
  repeat 10
    if value => i
      print(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      print("0")
    i /= 10

PRI print_decFixed(value,width) | i, p, f, a, b, c
'' Print a decimal number of fixed width
  p := @a
  f~
  if value < 0
    -value
    byte[p++] := "-"
  i := 1_000_000_000
  repeat 10
    if value => i
      byte[p++] := value / i + "0"
      value //= i
      f~~
    elseif f or i == 1
      byte[p++] := "0"
    i /= 10
  byte[p]~
  i := strsize(@a)
  if i < width
    repeat width - i
      print(" ")
  print_str(@a)
  
PRI print_hex(value, digits)
'' Print a hexadecimal number
  value <<= (8 - digits) << 2
  repeat digits
    print(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))

PRI print_bin(value, digits)
'' Print a binary number
  value <<= 32 - digits
  repeat digits
    print((value <-= 1) & 1 + "0")

PRI print(c) | i, k

'' Print a character
''
''       $0D = new line
''  $20..$FF = character
''      $100 = clear screen
''      $101 = home
''      $108 = backspace
''$110..$11F = select color

  case c
    $0D:                'return?
      newline

    $20..$FF:           'character?
      k := color << 1 + c & 1
      i := $8000 + (c & $FE) << 6 + k
      array.word[row * cols + col] := i
      array.word[(row + 1) * cols + col] := i | $40
      if ++col == cols
        newline

    $100:               'clear screen?
      wordfill(@array, spacetile, tiles)
      col := row := 0

    $101:               'home?
      col := row := 0

    $108:               'backspace?
      if col
        col--

    $110..$11F:         'select color?
      color := c & $F


PRI newline | i

  col := 0
  if (row += 2) == rows
    row -= 2
    'scroll lines
    repeat i from 0 to rows-3
      wordmove(@array.word[i*cols], @array.word[(i+2)*cols], cols)
    'clear new line
    wordfill(@array.word[(rows-2)*cols], spacetile, cols<<1)
      

DAT  
vgacolors long

  long $3C043C04       'lt grey on dk grey
  long $3C3C0404
  long $C000C000       'red
  long $C0C00000
  long $30003000       'green
  long $30300000
  long $0C000C00       'blue
  long $0C0C0000
  long $FC00FC00       'white
  long $FCFC0000
  long $FF80FF80       'red/white
  long $FFFF8080
  long $FF20FF20       'green/white
  long $FFFF2020
  long $FF28FF28       'cyan/white
  long $FFFF2828
  long $C0408080       'redbox
  long $3010F020       'greenbox
  long $3C142828       'cyanbox
  long $FC54A8A8       'greybox
  long $3C14FF28       'cyanbox+underscore
  long $F030C050       'graphics colors